\name{ProbBins}
\alias{ProbBins}
\title{
Computes probabilistic information of a forecast relative to a threshold or a quantile. 
}
\description{
Compute probabilistic bins of a set of forecast years ('fcyr') relative to the forecast climatology over the whole period of anomalies, optionally excluding the selected forecast years ('fcyr') or the forecast year for which the probabilistic bins are being computed (see 'compPeriod').
}
\usage{
ProbBins(ano, fcyr, thr, quantile = TRUE, posdates = 3, posdim = 2, compPeriod = "Full period")
}
\arguments{
  \item{ano}{
Array of anomalies from Ano().\cr
Must be of dimension (nexp/nobs, nmemb, nsdates, nleadtime, nlat,  nlon)
  }
  \item{fcyr}{
Indices of the forecast years of the anomalies of which to compute the probabilistic bins.\cr
Ex: c(1:5), c(1, 4) or 4.
  }
  \item{thr}{
Values used as thresholds to bin the anomalies.
  }
  \item{quantile}{
If quantile is TRUE (default), the threshold ('thr') are quantiles.\cr
If quantile is FALSE the thresholds ('thr') introduced are the absolute thresholds of the bins.
  }
  \item{posdates}{
Position of the dimension in 'ano' that corresponds to the start dates (default = 3).
  }
  \item{posdim}{
Position of the dimension in 'ano' which will be combined with 'posdates' to compute the quantiles (default = 2, ensemble members).
  }
  \item{compPeriod}{
Three options: "Full period"/"Without fcyr"/"cross-validation" (The probabilities are computed with the terciles based on ano/ano with all 'fcyr's removed/cross-validation). The default is "Full period".
  }
}
\value{
Matrix with probabilistic information and dimensions:\cr
c(length('thr'+1), nfcyr, nmemb/nparam, nmod/nexp/nobs, nltime, nlat, nlon)\cr
The values along the first dimension take values 0 or 1 depending on which of the 'thr'+1 cathegories the forecast/observation at the corresponding grid point, time step, member and starting date belongs to.
}
\examples{
sampleMap <- Load('tos', c('i00k'), c('ERSST'), c('19851101', '19901101', '19951101', '20001101', '20051101'), nleadtime = 124, leadtimemin = 1, leadtimemax = 60, sampleperiod = 1, output = 'lonlat', latmin = 30, latmax = 45, lonmin = 0, lonmax = 40)

clim <- Clim(sampleMap$mod, sampleMap$obs)
ano_exp <- Ano(sampleMap$mod, clim$clim_exp)
PB <- ProbBins(ano_exp, fcyr = 3, thr = c(1/3, 2/3), quantile = TRUE, posdates = 3, posdim = 2)
}
\author{
History:
0.1  -  2013  (F.Lienert)  -  Original code
1.0  -  2014-03  (N. Gonzalez and V.Torralba, \email{veronica.torralba@ic3.cat})  -  Debugging and enhancement
}
\keyword{datagen}
